%cmg_adj_mat  construct a adjacency matrix from cog map data
%
% Input:
%   P = predicate cell array
%   C = consequent cell array
%   inference_count = 1 if inference count is desired in A
%                     0 if 0-1 matrix is desired
%
% Output:
%   A = adjacency matrix with counts
%   class_list = cell array of all classes
%   class_map = containers.Map() object from class to index
%   is_dag = 1 if A represents a dag
%            0 if A contains cycles
%

function [A class_list class_map is_dag] = cmg_adj_mat(P,C,inference_count)

% expand the cmg
[PE CE] = cmg_expand(P,C);
num_inferences = length(CE);

% get initial class_list
class_list = unique([PE; CE]);
num_classes = length(class_list);

% build initial class_map
class_map = cmg_mk_class_map(class_list);

% build adj matrix with counts
A = zeros(num_classes,num_classes);
for i = 1:num_inferences
  A(class_map(PE{i}),class_map(CE{i})) = A(class_map(PE{i}),class_map(CE{i})) + 1;
end

% attempt to get topological order
topo_perm = topological_order(sparse(A));

if isempty(topo_perm)
  % matrix contains cycles
  is_dag = 0;
else
  % matrix is a dag
  is_dag = 1;
  
  % shuffle all output data to topological order
  A = A(topo_perm,topo_perm);
  class_list = class_list(topo_perm);
  class_map = cmg_mk_class_map(class_list);
  
end

if isempty(inference_count) || ~inference_count
  A(A~=0) = 1;
end

end

function class_map = cmg_mk_class_map(class_list)
%cmg_mk_class_map  construct class_map from class_list
%
% Given a cell array of unique strings returns a containers.Map() object
% from string to integer index in the original array.
%

class_map = containers.Map();

for i = 1:length(class_list)
  class_map(class_list{i}) = i;
end

end