%cmg_bnet  construct bnet object from cmg struct
%
% This is a complicated bit of code that translates the cmg struct into a
% form for processing by the Matlab Baysian Network Toolbox.  This code
% does not perform any computations.  It only translates one form of data
% to another.  The logical connections from the input cmg are exactly the
% same.
% 
% This code is a bit complicated because several transformations are
% required.
%
% Input:
%  G = a cognitive map graph structure as produced by cmg_struct.m
%  default_state = if 0, default state of pure predicates is off
%                  if 1, default state of pure predicates is on
%                  defaults to 0
%
% Output:
%  bnet = a baysian network object for use with bnt
%  logic_map = a Map object from class name to logical statement string
%              the logical statements refer to the parents in bnet.parents
%              these logical statements are used to construct the CPTs in bnet
%              they are returned so that the user can check correctness
%

function [bnet logic_map] = cmg_bnet(G,default_state)

% handle optional default_state
if nargin < 2 || isempty(default_state)
  default_state = 0;  
end

% check to make sure G is a dag
if ~G.is_dag
  error('cmg_bnet:not_dag','map: %s is not a dag.',G,name);
end

% all nodes are binary, and thus have size 2
node_sizes = 2*ones(G.n,1);

% create the initial bnet object
bnet = mk_bnet(G.A,node_sizes,'discrete',1:G.n,'names',G.class_list);
% bnet is not yet loaded with CPTs.  The CPTs are tables that indiciate the
% logical relationships between variables

% We need to "compress" the input P and C data in order to look up all
% predicate sets given a consequent.  For example, if the data contains
%  P = {{'A'} {'B' 'C'}}; C = {'D' 'D'};
% We want CC to be a map that gives
%  CC('D') -> {{'A'} {'B' 'C'}}
% This is needed so that we can easily look up all predicate sets for any
% given class.  If a class is a pure predicate (has no parents) then the
% result is an empty cell.
CC = cmg_compress(G);

% For each class we need to convert the predicate set in string format to a
% predicate set in index format.  The index format is particular to the
% bnet object so this must be passed in.
CN = cmg_str2num(CC,G,bnet);

% Finally for each class we create a logical statement string that will be
% used by bnt to construct the conditional probability table (CPT) used to
% define the relationships between variables.
logic_map = cmg_logic_str(CN);

%keyboard

% Here we loop through all variables and construct the CPTs using the
% logical strings.  If a class is a pure predicate (has no parents), we set
% its default state to 'off' or 'false'.  Otherwise the logical statement
% is used to construct the CPT.
for i = 1:G.n
  % get the name of the current class
  class_str = G.class_list{i};
  % get the logic string for this class
  logic_str = logic_map(class_str);
  if isempty(logic_str)
    % if the logic string is empty, this class has no parents.  the default
    % state is set to 'off' or 'false'
    if default_state == 0
      bnet.CPD{i} = tabular_CPD(bnet,i,[1 0]);
    elseif default_state == 1
      bnet.CPD{i} = tabular_CPD(bnet,i,[0 1]);
    end
  else
    % here the logic string is not empty.  the function boolean_CPD
    % constructs the CPT using the logic string
    bnet.CPD{i} = boolean_CPD(bnet,i,'inline',inline(logic_str));
  end
end

end


function CC = cmg_compress(G)
%cmg_compress  construct a map from consequents to predicate set
%
% Input:
%  G = cognitive map graph
%
% Output:
%  CC = Map from class string to predicate set cell array
%

% create empty Map object
CC = containers.Map();

% initialize map with empty cells
for i = 1:G.n
  CC(G.class_list{i}) = {};
end

% loop through entire inference list
for i = 1:length(G.P)
  CC(G.C{i}) = [CC(G.C{i}) G.P(i)];
end

end

function CN = cmg_str2num(CC,G,bnet)
%cmg_str2num  convert strings to indices in CC
%
% Input:
%  CC = Map from class string to predicate string set cell array
%  G = cognitive map graph structure
%  bnet = bnet object from baysian network toolbox
%
% Output:
%  CN = Map from class string to predicate index set cell array
%

% initialize empty map
CN = containers.Map();
for i = 1:G.n
  CN(G.class_list{i}) = {};
end

% loop through all classes
for i = 1:G.n

  % get the class string
  class_str = G.class_list{i};
  
  % get the set of parents for this class
  parent_set = G.class_list(bnet.parents{i});
  
  % construct map from parent string to index
  parent_map = cmg_mk_map(parent_set);
  
  % get the set of predicates
  pred_set = CC(class_str);
  
  % translate predicate sting set to indices
  pred_idx_set = cmg_str2num_helper(pred_set,parent_map);
  
  % store the indicies
  CN(class_str) = pred_idx_set;
  
end

end

function idx_cell = cmg_str2num_helper(str_cell,str_map)
%cmg_str2num_helper  helper function for cmg_num2str
%
% This function translates a cell of class stings to a cell of class
% indicies.
%
% Input:
%  str_cell = cell array of strings (this is a cell of cells of strings)
%  str_map = containers.Map object which gives the index for a string
%
% Output:
%  idx_cell = cell array of indices (it is a cell of cells of indices)
%             note the indices are returned as strings (not numeric values)
%

% initialize empty cell
idx_cell = {};

% get length of input
m = length(str_cell);

% loop through predicate sets
for i = 1:m
  
  % get current predicate set
  x = str_cell{i};
  
  % get length of current predicate set
  n = length(x);
  
  % initialize current index set
  y = {};
  
  % loop through each predicate in this set
  for j = 1:n
    y{j} = num2str(str_map(x{j}));
  end
  
  % add this index set to the output cell
  idx_cell{i} = y;
  
end

end

function CL = cmg_logic_str(CN)
%cmg_logic_str  build logic string from cell array of predicate index sets
%
% Input:
%  CN = Map from class string to predicate index set cell array
%
% Output:
%  CL = Map from class string to logical string
%

% initialize output map
CL = containers.Map();

% get the list of keys from the input
CN_keys = keys(CN);

% loop through keys
for i = 1:length(CN_keys)
  % translate cell arrays of indices to logical string
  CL(CN_keys{i}) = cmg_logic_str_helper(CN(CN_keys{i}));
end

end

function logic_str = cmg_logic_str_helper(logic_cell)
%cmg_logic_str_helper  convert cell array of indices to logical string
%
% Input:
%  logic_cell = cell array of predicate indices
%
% Output:
%  logic_str = logic string
%
% Example:
%  logic_cell = {{'1'} {'2' '3'}};
%  logic_str = cmg_logic_str(logic_cell);
%  logic_str -> '(x(1))|(x(2)&x(3))'
%

% initialize logic_str with empty string
logic_str = '';

% get length of input
m = length(logic_cell);

% loop through each predicate index set
for i = 1:m
  
  % get the current predicate set
  logic_item = logic_cell{i};
  
  % get length of current predicate set
  n = length(logic_item);
  
  logic_str = [logic_str '('];
  for j = 1:n
    logic_str = [logic_str 'x(' logic_item{j} ')&'];
  end
  logic_str = [logic_str(1:end-1) ')|'];
end
logic_str = logic_str(1:end-1);

end

function M = cmg_mk_map(L)
%cmg_mk_map  build map from strings to integers from cell array of strings
%
% Give a cell array of strings, this method returns a containers.Map object
% that maps from string to index.  This is useful when needing to find the
% index of some string.
%
% Input:
%  L = cell array of strings
%
% Output:
%  M = containers.Map object where M(L{i}) returns i.
%

% initialize empty map
M = containers.Map();

% loop through each class adding the string and index to the map
for i = 1:length(L)
  M(L{i}) = i;
end

end