%cmg_sample  generate a sample from a cmg given evidence
%
% Runs a cognitive map given input decision classes to assert or deny.  Returns
% information on the set of classes and inferences that are activated.
%
% Input:
%  G = cmg struct
%  bnet = cmg bnet object
%  assert_cell = cell array of class names to assert
%  deny_cell = cell array of class names to deny
%  decision_set = cell array of class names representing decisions
%
% Output:
%  active_cons = cell array of active pure consequents, filtered by
%                decision_set if specified
%  active_class = cell array of all active classes
%  result = direct result from sample_bnet
%  active_lx = logical index of active classes
%  inference_lx = logical index of active inferences
%

function [active_cons active_class result active_lx inference_lx] = ...
      cmg_sample(G,bnet,assert_cell,deny_cell,decision_set)

  if nargin < 5 || isempty(decision_set)
    decision_set = {};
  end
  
  % set initial evidence to off
  evidence = cell(G.n,1);

  % assert desired classes
  assert_cell = intersect(assert_cell,G.class_list);
  for i = 1:length(assert_cell)
    evidence{G.class_map(assert_cell{i})} = 2;
  end

  % deny desired classes
  deny_cell = intersect(deny_cell,G.class_list);
  for i = 1:length(deny_cell)
    evidence{G.class_map(deny_cell{i})} = 1;
  end

  % sample the bnet
  result = sample_bnet(bnet,'evidence',evidence);

  % compute logical index of activated classes
  active_lx = cell_eq(result,2);

  % obtain set of active pure consequents
  active_cons = G.class_list(active_lx & G.cons_lx);

  % if a decision set is specified, filter list of active constraints
  if ~isempty(decision_set)
    active_cons = intersect(active_cons,decision_set);
  end
  
  % obtain list of active classes
  active_class = G.class_list(active_lx);

  % compute logical index of active inferences
  inference_lx = cmg_active_inference(G.P,active_class);

end

function y = cell_eq(x,val)
%cell_eq  The eqivalent of "y = x == val" for cells
%

  x_size = size(x);
  n = prod(x_size);
  y = zeros(x_size);

  for i = 1:n
    
    if x{i} == val
      y(i) = 1;
    end
    
  end

  y = logical(y);

end