%cmg_struct  generate cmg struct from predicate and consequent cells
%
% Input:
%  Pstr = predicate cell array
%  C = consequent cell array
%  name = name of the map represented by the struct
%  inference_count = 0 if adj matrix just indicates connections (default)
%                    1 if adj matrix counts the number of inferences
%
% Output:
%  G = cmg struct
%
% Struct fields:
%  name = name for the cmg, defaults to cmg
%  P = predicate cell array
%  Pstr = cell array of predicate strings
%  C = consequent cell array
%  inf_str = cell array of inference strings
%  inf_cnt = count of inferences
%  A = adjacency matrix
%  is_dag = 1 if A represents a directed acyclic graph, 0 otherwise
%  n = number of classes
%  class_list = cell array of classes, order determines index in adjacency
%                matrix
%  class_map = containers.Map of classes
%  pred_lx = logical index of pure predicate classes
%  cons_lx = logical index of pure consequent classes
%  intm_lx = logical index of intermediate classes
%

function G = cmg_struct(Pstr,C,name,inference_count)

% check optional name input
if nargin < 3
  name = 'cmg';
end

% check inference count input
if nargin < 4 || isempty(inference_count)
  inference_count = 0;
end

% orient arrays
Pstr = Pstr(:);
C = C(:);

% get length and check sizes
np = length(Pstr);
nc = length(C);

% get predicate cell
P = cmg_Pstr2Pcell(Pstr);

if np ~= nc
  error('cmg_struct:input','Input cell arrays P and C must be same length.');
end

% construct the adjacency matrix
[A class_list class_map is_dag] = cmg_adj_mat(P,C,inference_count);

% get number of classes
n = length(class_list);

% compute index arrays
pred_lx = sum(A,1)' == 0;
cons_lx = sum(A,2) == 0;
intm_lx = ~pred_lx & ~cons_lx;

% construct the struct
G.name = name;
G.P = P;
G.Pstr = Pstr;
G.C = C;
G.inf_str = cmg_inf_str(G.Pstr,G.C);
G.inf_cnt = np;
G.A = A;
G.is_dag = is_dag;
G.n = n;
G.class_list = class_list;
G.class_map = class_map;
G.pred_lx = pred_lx;
G.cons_lx = cons_lx;
G.intm_lx = intm_lx;

end
