%cmg_viz  generate a visualization of a congitive map graph
%
% This function produces either a pdf or a graphviz dot file of a cognitive
% map structure.
%
% System requirements:
%  graph_to_dot.m: http://www.mathworks.com/matlabcentral/fileexchange/4518
%  graphviz utilities: http://www.graphviz.org/
%
% Usage:
%  cmg_viz(G)
%  cmg_viz(G,filename)
%  cmg_viz(G,filename,dot_flag)
%  [status message] = cmg_viz(G,filename,dot_flag)
%
% Input:
%  G = cmg struct (uses fields A and class_list)
%  node_state = cell array of with elements in {1,2}, has length G.n
%  edge_weight = 1 if edge weights should be printed, 0 otherwise
%  filename = output file name (optional, defaults to 'tmp')
%  dot_flag = 1 if a graphviz dot file is desiered (optional, defaults to 0)
%
% Output:
%  status = status flag from system call to graphviz 'dot' util
%  message = system message from call to graphviz 'dot' util
%

function [status message] = cmg_viz(G,node_state,edge_weight,filename,dot_flag)

% handle optional node_state input
if nargin < 2 || isempty(node_state)
  node_state = cell(G.n);
  for i = 1:G.n
    node_state{i} = 1;
  end
end

% handle options edge_count input
if nargin < 3 || isempty(edge_weight)
  edge_weight = 0;
end

% handle optional filename input
if nargin < 4 || isempty(filename)
  filename = 'tmp';
end

% handle optional dot_flag input
if nargin < 5 || isempty(dot_flag)
  dot_flag = 0;
end

% set initial status and message output
status = 0;
message = '';

% append suffix to dot filename
dot_file = [filename '.dot'];

% get the node colors
node_color = cmg_state2color(node_state);

% generate arc_label cell array of strings
%arc_label = cmg_num2strcell(G.A);
% produce the graphviz dot file
%graph_to_dot(G.A,'node_label',G.class_list,'arc_label',arc_label,'filename',dot_file);

if ~edge_weight
  graph_to_dot(G.A, ...
    'node_label',G.class_list, ...
    'node_color',node_color, ...
    'graph_label',G.name, ...
    'filename',dot_file);
else
  Acell = cmg_edge_label(G.A);
  graph_to_dot(G.A, ...
    'arc_label',Acell, ...
    'node_label',G.class_list, ...
    'node_color',node_color, ...
    'graph_label',G.name, ...
    'filename',dot_file);
end
  
% function produces pdf by default
if ~dot_flag
  % pdf is requested
  
  % append file type suffix to filename
  pdf_file = [filename '.pdf'];
  
  % generate graphviz command
  dot_command = ['dot ' dot_file ' -Tpdf -o' pdf_file];
  
  % execute graphviz command
  [status message] = system(dot_command);

  % remove the temporary dot file
  if isunix
    system(['rm ' dot_file]);
  elseif ispc
    system(['del ' dot_file]);
  end
end

end

function color_cell = cmg_state2color(state_cell)

n = length(state_cell);
color_cell = cell(n,1);

for i = 1:n
  if state_cell{i} == 1
    color_cell{i} = 'black';
  elseif state_cell{i} == 2
    color_cell{i} = 'red';
  else
    color_cell{i} = '';
  end
end

end

function Acell = cmg_edge_label(A)
sz = size(A);
Acell = cell(sz);
for i = 1:prod(sz)
  Acell{i} = num2str(A(i));
end
end