%cmg_viz_2  generate a visualization of a congitive map graph
%
% This function produces either a pdf or a graphviz dot file of a cognitive
% map structure.
%
% System requirements:
%  graph_to_dot.m: http://www.mathworks.com/matlabcentral/fileexchange/4518
%  graphviz utilities: http://www.graphviz.org/
%
% Usage:
%  cmg_viz(G)
%  cmg_viz(G,filename)
%  cmg_viz(G,filename,dot_flag)
%  [status message] = cmg_viz(G,filename,dot_flag)
%
% Input:
%  G = cmg struct (uses fields A and class_list)
%  node_state = cell array of with elements in {1,2}, has length G.n
%  edge_weight = 1 if edge weights should be printed, 0 otherwise
%  filename = output file name (optional, defaults to 'tmp')
%  dot_flag = 1 if a graphviz dot file is desiered (optional, defaults to 0)
%
% Output:
%  status = status flag from system call to graphviz 'dot' util
%  message = system message from call to graphviz 'dot' util
%

function [status message] = cmg_viz_2(...
    filename, ...
    G, ...
    disp_lx, ...
    act_set, ...
    on_set, ...
    off_set, ...
    edge_weight, ...
    dot_flag, ...
    line_break)
  
  % handle optional line_break input
  if nargin < 9 || isempty(line_break)
    line_break = 0;
  end
  
  % handle optional dot_flag input
  if nargin < 8 || isempty(dot_flag)
    dot_flag = 0;
  end
  
  % handle options edge_count input
  if nargin < 7 || isempty(edge_weight)
    edge_weight = 0;
  end
  
  % optional off_set input
  if nargin < 6 || isempty(off_set)
    off_set = {};
  end
  
  % optional on_set input
  if nargin < 5 || isempty(on_set)
    on_set = {};
  end
  
  % optional on_set input
  if nargin < 4 || isempty(act_set)
    on_set = {};
  end
  
  % optional act_set input
  if nargin < 3 || isempty(disp_lx)
    disp_lx = true(G.n,1);
  end
  
  % add classes in off_set to disp_lx
  if ~isempty(off_set)
    for i = 1:G.n
      if ismember(G.class_list{i},off_set)
        disp_lx(i) = true;
      end
    end
  end
  
  % hack to not display title!
  G.name = '';
  
  % restrict adj matrix and class list
  A = G.A(disp_lx,disp_lx);
  class_list = G.class_list(disp_lx);
  num_nodes = length(class_list);
  
  % create color cell
  node_color = cell(num_nodes,1);
  for i = 1:num_nodes
    node_color{i} = 'black';
    if ismember(class_list{i},act_set)
      node_color{i} = 'blue';
    end
    if ismember(class_list{i},on_set)
      node_color{i} = 'green';
    end
    if ismember(class_list{i},off_set)
      node_color{i} = 'red';
    end
  end
  
  % set initial status and message output
  status = 0;
  message = '';
  
  % append suffix to dot filename
  dot_file = [filename '.dot'];
  
  % change the labels
  if line_break
    for i = 1:length(class_list)
      class_list{i} = strrep(class_list{i},'-','\n');
    end
  end
  
  % produce the dot file
  if ~edge_weight
    graph_to_dot(A, ...
      'node_label',class_list, ...
      'node_color',node_color, ...
      'graph_label',G.name, ...
      'filename',dot_file);
  else
    Acell = cmg_edge_label(A);
    graph_to_dot(A, ...
      'arc_label',Acell, ...
      'node_label',class_list, ...
      'node_color',node_color, ...
      'graph_label',G.name, ...
      'filename',dot_file);
  end
  
  % function produces pdf by default
  % append file type suffix to filename
  pdf_file = [filename '.pdf'];
  
  if ismac
    eps_file = [filename '.eps'];
    % generate commands
    eps_command = ['dot ' dot_file ' -Teps -o' eps_file];
    pdf_command = ['epstopdf ' eps_file];
    % execute commands
    [status message] = system(eps_command);
    [status message] = system(pdf_command);
    % remove temp files
    if ~dot_flag
      system(['rm ' dot_file]);
    end
    system(['rm ' eps_file]);
  else
    % generate command
    pdf_command = ['dot ' dot_file ' -Tpdf -o' pdf_file];
    % execute command
    [status message] = system(pdf_command);
    % remove the temporary dot file
    if ~dot_flag
      if isunix
        system(['rm ' dot_file]);
      elseif ispc
        dot_file = strrep(dot_file,'/','\');
        system(['del ' dot_file]);
      end
    end
  end
  
end

function Acell = cmg_edge_label(A)
  sz = size(A);
  Acell = cell(sz);
  for i = 1:prod(sz)
    Acell{i} = num2str(A(i));
  end
end